// compareGaborMex.cpp
// To compile, run at the command prompt:
// mex compareGaborMex.cpp

#include <math.h>
#include "mex.h"

// Inputs: gaborQuery,allGabor,w,Sx,Sy,cellSize,stackNdx
void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
  if(nrhs != 6) {
    mexErrMsgTxt("Error: 6 args. needed.");
    return;
  }

  // Get inputs:
  float *gaborQuery = (float*)mxGetData(prhs[0]);
  float *allGabor = (float*)mxGetData(prhs[1]);
  float *w = (float*)mxGetData(prhs[2]);
  int *Sx = (int*)mxGetData(prhs[3]);
  int *Sy = (int*)mxGetData(prhs[4]);
  int *stackNdx = (int*)mxGetData(prhs[5]);

  // Get allGabor dimensions:
  const mwSize *dims = mxGetDimensions(prhs[1]);
  int Gnrows = dims[0];
  int Gncols = dims[1];
  int nFilters = dims[2];
  int Gnimg = dims[3];

  // Get number of grid cells:
  int Nx = mxGetNumberOfElements(prhs[3])-1;
  int Ny = mxGetNumberOfElements(prhs[4])-1;

  // Allocate outputs:
  plhs[0] = mxCreateNumericMatrix(1,Gnimg,mxSINGLE_CLASS,mxREAL);
  float *d = (float*)mxGetData(plhs[0]);

  // Allocate temporary memory:
  int featureLen = nFilters*Nx*Ny;
  float *g = (float*)mxCalloc(featureLen,sizeof(float));
  float *gQuery = (float*)mxCalloc(featureLen,sizeof(float));

  // Initialize output matrix to be all zeros:
  for(int i = 0; i < Gnimg; i++) d[i] = 0;

  // Initialize gQuery to be all zeros:
  for(int i = 0; i < featureLen; i++) gQuery[i] = 0;

  // Compute gist feature for query image:
  for(int xx = 0; xx < Nx; xx++) {
//     int Nsx = Sx[xx+1]-Sx[xx];
    for(int yy = 0; yy < Ny; yy++) {
//       int Nsy = Sy[yy+1]-Sy[yy];
      for(int ff = 0; ff < nFilters; ff++) {
	for(int sx = Sx[xx]; sx < Sx[xx+1]; sx++) {
	  for(int sy = Sy[yy]; sy < Sy[yy+1]; sy++) {
	    gQuery[yy+xx*Ny+ff*Nx*Ny] += w[sy+sx*Gnrows+ff*Gnrows*Gncols]*gaborQuery[sy+sx*Gnrows+ff*Gnrows*Gncols];
// 	    gQuery[yy+xx*Ny+ff*Nx*Ny] += w[sy+sx*Gnrows]*gaborQuery[sy+sx*Gnrows+ff*Gnrows*Gncols]/Nsx/Nsy;
	  }
	}
      }
    }
  }

  for(int n = 0; n < Gnimg; n++) {
    // Clear gist feature:
    for(int i = 0; i < featureLen; i++) g[i] = 0;

    // Get gist feature from allGabor:
    for(int xx = 0; xx < Nx; xx++) {
//       int Nsx = Sx[xx+1]-Sx[xx];
      for(int yy = 0; yy < Ny; yy++) {
// 	int Nsy = Sy[yy+1]-Sy[yy];
	for(int ff = 0; ff < nFilters; ff++) {
	  for(int sx = Sx[xx]; sx < Sx[xx+1]; sx++) {
	    for(int sy = Sy[yy]; sy < Sy[yy+1]; sy++) {
	      g[yy+xx*Ny+ff*Nx*Ny] += w[sy+sx*Gnrows+ff*Gnrows*Gncols]*allGabor[sy+sx*Gnrows+ff*Gnrows*Gncols+n*Gnrows*Gncols*nFilters];
// 	      g[yy+xx*Ny+ff*Nx*Ny] += w[sy+sx*Gnrows]*allGabor[sy+sx*Gnrows+ff*Gnrows*Gncols+n*Gnrows*Gncols*nFilters]/Nsx/Nsy;
	    }
	  }
	}
      }
    }

    // Compute L1 distance between gist feature and query image:
    for(int i = 0; i < featureLen; i++) d[n] += fabs(gQuery[i]-g[i]);
//     for(int i = 0; i < featureLen; i++) d[n] += pow(gQuery[i]-g[i],2);
//     for(int i = 0; i < featureLen; i++) d[n] += w[i]*fabs(gQuery[i]-g[i]);
  }

  // Free temporary memory:
  mxFree(g);
  mxFree(gQuery);

  return;
}

// // Inputs: allGabor,startX,startY,stackNdx,gQuery,numberBlocksY,numberBlocksX,featureCellSize,cellSize
// // Outputs: d
// void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
//   if(nrhs != 10) {
//     mexErrMsgTxt("Error: 10 args. needed.");
//     return;
//   }

//   // Get inputs:
//   float *allGabor = (float*)mxGetData(prhs[0]);
//   double *startX = mxGetPr(prhs[1]);
//   double *startY = mxGetPr(prhs[2]);
//   double *stackNdx = mxGetPr(prhs[3]);
//   float *gQuery = (float*)mxGetData(prhs[4]);
//   int numberBlocksY = (int)*mxGetPr(prhs[5]);
//   int numberBlocksX = (int)*mxGetPr(prhs[6]);
//   int featureCellSize = (int)*mxGetPr(prhs[7]);
//   int cellSize = (int)*mxGetPr(prhs[8]);
//   double *weightMask = (double*)mxGetPr(prhs[9]);

//   // Get allGabor dimensions:
//   const mwSize *dims = mxGetDimensions(prhs[0]);
//   int Gnrows = dims[0];
//   int Gncols = dims[1];
//   int nFilters = dims[2];
//   int Gnimg = dims[3];

//   // Get stack dimensions:
//   dims = mxGetDimensions(prhs[1]);
//   int nrows = dims[0];
//   int ncols = dims[1];
//   int nimg = dims[2];

//   int featureLen = numberBlocksX*numberBlocksY*nFilters;
//   featureCellSize /= cellSize;

//   // Allocate outputs:
//   plhs[0] = mxCreateNumericArray(3,dims,mxSINGLE_CLASS,mxREAL);
//   float *d = (float*)mxGetData(plhs[0]);

//   // Allocate temporary memory:
//   float *g = (float*)mxCalloc(featureLen,sizeof(float));

//   // Initialize output matrix to be all zeros:
//   for(int i = 0; i < nrows*ncols*nimg; i++) d[i] = 0;

//   // Loop through spatial locations:
//   for(int x = 0; x < ncols; x++) {
//     for(int y = 0; y < nrows; y++) {
//       // Loop through stack images:
//       for(int n = 0; n < nimg; n++) {
// 	// Get start coordinates for allGabor:
// 	int xx = (int)startX[y+x*nrows+n*nrows*ncols]-1;
// 	int yy = (int)startY[y+x*nrows+n*nrows*ncols]-1;
// 	int nn = (int)stackNdx[y+x*nrows+n*nrows*ncols]-1;
// 	xx /= cellSize;
// 	yy /= cellSize;

// 	// Clear gist feature:
// 	for(int i = 0; i < featureLen; i++) g[i] = 0;

// 	// Get gist feature from allGabor:
// 	for(int fx = 0; fx < featureCellSize; fx++) {
// 	  for(int fy = 0; fy < featureCellSize; fy++) {
// 	    for(int gx = 0; gx < numberBlocksX; gx++) {
// 	      for(int gy = 0; gy < numberBlocksY; gy++) {
// 		for(int gf = 0; gf < nFilters; gf++) {
// 		  g[gy+gx*numberBlocksY+gf*numberBlocksX*numberBlocksY] += allGabor[(yy+fy+gy*featureCellSize)+(xx+fx+gx*featureCellSize)*Gnrows+gf*Gnrows*Gncols+nn*Gnrows*Gncols*nFilters];
// 		}
// 	      }
// 	    }
// 	  }
// 	}

// 	// Normalize gist feature:
// 	for(int i = 0; i < featureLen; i++) g[i] /= featureCellSize*featureCellSize;

// 	// Compute L1 distance between gist feature and query image:
// 	for(int i = 0; i < featureLen; i++) d[y+x*nrows+n*nrows*ncols] += weightMask[i]*fabs(gQuery[y+x*nrows+i*nrows*ncols]-g[i]);
//       }
//     }
//   }

//   // Free temporary memory:
//   mxFree(g);

//   return;
// }
