function g = gistGabor_NxNy(img, wx,wy, G)
% 
% Input:
%   img = input image (it can be a block: [nrows, ncols, c, Nimages])
%   wx,wy = number of windows wy... in row direction, wx.. in column
%   direction
%   G = precomputed transfer functions
%
% Output:
%   g: are the global features = [Nfeatures Nimages], 
%                    Nfeatures = Nx*Ny*Nfilters*c

if ndims(img)==2
    c = 1; 
    N = 1;
end
if ndims(img)==3
    [nrows ncols c] = size(img);
    N = c;
end
if ndims(img)==4
    [nrows ncols c N] = size(img);
    img = reshape(img, [nrows ncols c*N]);
    N = c*N;
end

[ny nx Nfilters] = size(G);
W = wx*wy;
g = zeros([W*Nfilters N]);

%img = single(fft2(img)); 
img = (fft2(img)); 

k=0;
for n = 1:Nfilters
    ig = abs(ifft2(img.*repmat(G(:,:,n), [1 1 N])));    
    v = downN(ig, wx,wy);
    g(k+1:k+W,:) = reshape(v, [W N]);
    k = k + W;
    drawnow
end

if c == 3
    % If the input was a color image, then reshape 'g' so that one column
    % is one images output:
    g = reshape(g, [size(g,1)*3 size(g,2)/3]);
end


function y=downN(x, Nx,Ny)
% 
% averaging over non-overlapping square image blocks
%
% Input
%   x = [nrows ncols nchanels]
% Output
%   y = [Nx Ny nchanels] 
%
%  Note Nx .. columns
%       Ny .. rows

ny = fix(linspace(0,size(x,1),Ny+1));
nx = fix(linspace(0,size(x,2),Nx+1));
y  = zeros(Ny, Nx, size(x,3));
for xx=1:Nx
  for yy=1:Ny
    %v=mean(mean(x(nx(xx)+1:nx(xx+1), ny(yy)+1:ny(yy+1),:),1),2);
    %y(xx,yy,:)=v(:);
    
    v=mean(mean(x(ny(yy)+1:ny(yy+1), nx(xx)+1:nx(xx+1),:),1),2);
    y(yy,xx,:)=v(:);
    
  end
end
